@@ -16,7 +16,7 @@ String::Interpolate::Named - Interpolated named arguments in string
 
 =cut
 
-our $VERSION = '1.03';
+our $VERSION = '1.04';
 
 =head1 SYNOPSIS
 
@@ -109,6 +109,65 @@ When the value exceeds the number of elements in the list, an empty
 value is returned.
 When no element is selected the values are concatenated.
 
+=head2 Format modifiers
+
+A named variable may have I<format modifiers> attached to perform
+formatting operations on the substituted value.
+Format modifiers start with a colon C<:>.
+
+Assuming argument C<title> has the value C<"My Book">, then
+C<"%{title:lc}">
+will yield the title in lowercase C<"my book">.
+
+The following format modifiers are available:
+
+=over 6
+
+=item C<:lc>
+
+Yields the substituted value in all lower case.
+
+Using the example above, this will be C<"my book">.
+
+=item C<:uc>
+
+Yields the substituted value in all upper case.
+
+Using the example above, this will be C<"MY BOOK">.
+
+=item C<:ic>
+
+Yields the substituted value with initial caps, e.g. the first letter
+of each word is capitalized.
+
+Using the example above, this will be C<"My Book">.
+Indeed, no difference since the value is already correctly cased.
+
+To enforce lower case before applying initial case, use format modifiers
+C<:lc:ic>.
+
+=item C<:sc>
+
+Yields the substituted value with an initial cap and the rest lower case.
+
+Using the example above, this will be C<"My Book">.
+Again, no difference since the value already has an initial capital.
+
+To enforce lower case before applying initial case, use format modifiers
+C<:lc:sc>. Now the result will be C<"My book">.
+
+=item C<:%>I<fmt>
+
+Apply standard printf() formatting, e.g. C<%{key:%03d}> yields the numeric
+value of C<key> as a 3-digit string, adding leading zeroes if necessary.
+
+=back
+
+Note that, when combining formatting and conditional interpolation,
+you must check for the I<formatted> value:
+
+    "This takes %{days:%02d=01|%{} day|%{} days}"
+
 =head2 The Control Hash
 
 The interpolation process requires two parameters: a hash with
@@ -265,6 +324,7 @@ sub interpolate {
 		     (?<pre> .*? )
 		     \x{fddf}
 		     (?<key> $keypat )
+		     (?: : (?<fmt> .*? ) )?
 		     (?: (?<op> \= )
 			 (?<test> [^|}\x{fddf}]*) )?
 		     (?: \| (?<then> [^|}\x{fddf}]*  )
@@ -323,9 +383,9 @@ sub _interpolate {
 	( $key, $inx ) = ( $1, $2 );
     }
 
-    my $t = ref($m) eq 'CODE' ? $m->($key) : $m->{$key};
-    if ( defined $t ) {
-	$val = $t;
+    my $newval = ref($m) eq 'CODE' ? $m->($key) : $m->{$key};
+    if ( defined $newval ) {
+	$val = $newval;
 
 	if ( UNIVERSAL::isa( $val, 'ARRAY' ) ) {
 	    # 1, 2, ... selects 1st, 2nd value; -1 counts from end.
@@ -353,6 +413,23 @@ sub _interpolate {
     }
 
     my $subst = '';
+    for ( split( ':', $i->{fmt}//'' ) ) {
+	last unless defined $newval;
+	last unless my $fmt = $_;
+	if    ( $fmt eq 'lc' ) { $val = lc($val) }
+	elsif ( $fmt eq 'uc' ) { $val = uc($val) }
+	elsif ( $fmt eq 'sc' ) { $val = ucfirst($val) }
+	elsif ( $fmt eq 'ic' ) {
+	    $val = join('', map { ucfirst } (split( /(^|\s+|-)/, $val )));
+	}
+	elsif ( $fmt =~ /^%/ ) {
+	    no warnings qw(numeric);
+	    $val = sprintf( $fmt, $val );
+	}
+	else {
+	    Carp::croak("Invalid format code '$fmt'")
+	}
+    }
     if ( $i->{op} ) {
 	my $test = $i->{test} // '';
 	if ( $i->{op} eq '=' && $val eq $test ) {
@@ -373,6 +450,42 @@ sub _interpolate {
     return $subst;
 }
 
+use Scalar::Util qw(looks_like_number);
+sub qquote {
+    my ( $arg, $force ) = @_;
+    for ( $arg ) {
+	s/([\\\"])/\\$1/g;
+	s/([[:^print:]])/sprintf("\\u%04x", ord($1))/ge;
+	return $_ unless /[\\\s]/ || $force;
+	return qq("$_");
+    }
+}
+sub pv {
+    my $val   = pop;
+    my $label = pop // "";
+
+    my $suppressundef;
+    if ( $label =~ /\?$/ ) {
+	$suppressundef++;
+	$label = $';
+    }
+    if ( defined $val ) {
+	if ( looks_like_number($val) ) {
+	    $val = sprintf("%.3f", $val);
+	    $val =~ s/0+$//;
+	    $val =~ s/\.$//;
+	}
+	else {
+	    $val = qquote( $val, 1 );
+	}
+    }
+    else {
+	return "" if $suppressundef;
+	$val = "<undef>"
+    }
+    defined wantarray ? $label.$val : warn($label.$val."\n");
+}
+
 =head1 REQUIREMENTS
 
 Minimal Perl version 5.10.1.
@@ -399,7 +512,7 @@ Many of the existing template / interpolate / substitute modules.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2018,2019 Johan Vromans, all rights reserved.
+Copyright 2018,2025 Johan Vromans, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
